package application;

import javafx.application.Platform;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.ScrollPane;
import javafx.scene.control.TextArea;
import javafx.scene.control.TextField;
import javafx.scene.text.Font;

import java.io.PrintStream;
import java.net.URL;
import java.util.List;
import java.util.ResourceBundle;

import RalliJava.Henkilo;
import RalliJava.Ralli;
import RalliJava.Rekisteri;
import RalliJava.SailoException;
import fi.jyu.mit.fxgui.Dialogs;
import fi.jyu.mit.fxgui.ListChooser;
import fi.jyu.mit.fxgui.ModalController;
import fi.jyu.mit.fxgui.TextAreaOutputStream;

/**
 * @author idamk
 * @version 5.2.2025
 *
 */
public class RalliController implements Initializable {
    
    @FXML private TextField textrajaus; //hakutulosten rajaus'
    @FXML private ScrollPane panelHenkilo;
    @FXML private ListChooser<Henkilo> chooserHenkilot;
	
    /*
     * Alustaa vasemman reunan listan tyhjäksi.
     * TODO: lisätyn tiedot tulisi näkyviin keskelle tekstikenttään
     */
    @Override
   public void initialize(URL url, ResourceBundle bundle) {
        alusta();
    }
    
    
    /*
     * Avaa uuden ikkunan missä voi lisätä uuden henkilön tiedot ja tallentaa ne
     */
    @FXML private void handleUusiHenkilo() {
        uusiHenkilo();
    }
    
    
    @FXML private void handleUusiRalli() {
        //Dialogs.showMessageDialog("Lisää uuden rallin, ei vielä osaa :((");
        uusiRalli();
    }
    
    /*
     * Avaa ikkunan, missä pääsee muokkaamaan valitun RALLIN tietoja
     */
    @FXML private void handleMuokkaa() {
        //Dialogs.showMessageDialog("Muokkaa valittua henkilöä/rallia, ei osaa vielä :((");
        ModalController.showModal(RalliController.class.getResource("muokkausRalli.fxml"), "Muokkaa", null, "");
        //TODO: täytyy pystyä muokkaamaan myös henkilöitä
    }
    
    
    
    /*
     * 
     */
    @FXML private void handleRajaus() {
        Dialogs.showMessageDialog("Rajaa hakutuloksia, ei osaa vielä :((");
    }
    
    /*
     * Poistuu ikkunasta
     */
    @FXML private void handlePoistu() {
        //TODO:tallenna();
        Platform.exit();
    }


    
    /**
     * @return true
     */
    public boolean voikoSulkea() {
        //tallenna();
        return true;
    }
    
    
/*================================================================*/
 
     private Rekisteri rekisteri;
     private Henkilo henkiloKohdalla;
     private TextArea areaHenkilo = new TextArea(); //TODO poista lopuksi!!
     
     
     /**
      * @param rekisteri mitä käytetään
      */
      public void setRekisteri(Rekisteri rekisteri){
         this.rekisteri = rekisteri;
         naytaHenkilo();
      }
      
      /*
       * Näyttää valitun jäsenen tiedot väliaikaisesti
       * @param rekisteri 
       */ 
      private void naytaHenkilo() {
          henkiloKohdalla = chooserHenkilot.getSelectedObject();
          
          if(henkiloKohdalla == null) return; //jos ei oo valittuna ketään, poistutaan
          
          areaHenkilo.setText("");  //tyhjennetään entinen näkyviltä
          try (PrintStream out = TextAreaOutputStream.getTextPrintStream(areaHenkilo)) {
              
              out.println("======");
              henkiloKohdalla.tulosta(out);
              out.println("======");
              List<Ralli> rallit = rekisteri.annaRallit(henkiloKohdalla);
              for (Ralli ral: rallit)
                      ral.tulosta(out);
              out.println("======");
              
              //tulosta(out, henkiloKohdalla);
              //henkiloKohdalla.tulosta(out);
              //List<Ralli> rallit = rekisteri.annaRallit(henkiloKohdalla);
              //for(Ralli ral: rallit)
                  //ral.tulosta(out);
          }
      }
     
     /*
      * Alustaa vasemman teksti-ikkunan tyhjäksi
      * asettaa myös kuuntelijan listalle
      */
     private void alusta() {
         panelHenkilo.setContent(areaHenkilo);
         areaHenkilo.setFont(new Font("Courier New", 12));
         panelHenkilo.setFitToHeight(true);
         chooserHenkilot.clear();
         chooserHenkilot.addSelectionListener(e -> naytaHenkilo()); // e on lambdalauseke
         //TODO: saa uudet henkilöt näkymään listassa
     }
    
    
     /*
     * Haetaan henkilöt uudelleen
     * @param idnro henkilön tunniste
     */
     private void hae(int idnro) {
        chooserHenkilot.clear();
        
        int index = 0;
        for(int i = 0; i < rekisteri.getHenkiloita(); i++) {    //käydään läpi henkilöt
            Henkilo henkilo = rekisteri.annaHenkilo(i);         //indeksin perusteella käy läpi
            if (henkilo.getHenkilonTunnusNro() == idnro) index = i;     //samalla kun käy läpi henkilöitä, etsii oikeaa indeksiä
            chooserHenkilot.add(henkilo.getNimi(), henkilo);    // viimeinen kohta linkittää juuri tämän henkilön chooseriin
        }
        chooserHenkilot.setSelectedIndex(index); 
     }

    

    
    private void uusiHenkilo() {
        Henkilo uusi = new Henkilo();
        uusi.rekisteroi(); //TODO rekisteröinti vasta kun käyttäjä tallentaa uuden tiedot
        uusi.vastaahlo(); //TODO että käyttäjä saa täyttää itse
        //uusi.tulosta(System.out);
        try {
            rekisteri.lisaa(uusi);
        } catch (SailoException e) { //catch koska määrtelty että rekisteri saattaa heittää poikkeuksen
            Dialogs.showMessageDialog("Ongelmia uuden luomisessa " + e.getMessage());
        }
        hae(uusi.getHenkilonTunnusNro()); 
    }
    
    
    private void uusiRalli() {
        henkiloKohdalla = chooserHenkilot.getSelectedObject();
        if (henkiloKohdalla == null) return; // jos ei olla minkään jäsenen kohdalla
        //Chooserilta voi kysyä minkä henkilön kohdalla ollaan
        //Henkilo hlovalittuna = chooserHenkilot.getSelectedObject();
        Ralli ral = new Ralli();
        ral.rekisteroi();
        ral.vastaaRalli(henkiloKohdalla.getHenkilonTunnusNro());
        rekisteri.lisaa(ral);
        hae(henkiloKohdalla.getHenkilonTunnusNro());
        //TODO ei lisää rallia mutta ei anna erroria??? 
    }
    
    


    

   

}
